/* Multiset.java
 * CS193J Winter 1999
 */
 
import java.util.*;

/**
 * The <code>Multiset</code> class implements an unordered collection of 
 * elements using an efficient representation for duplicates. Elements that
 * are stored in a Multiset must properly respond to the equals() method
 * in order to allow correct coalescing of duplicates.
 * <P>
 * There is no element ordering maintained within a multiset.
 * The Multiset {B,C,B,A} is considered the same as {C,B,B,A} or {A,B,C,B}.
 * The implementation is free to arrange the elements in whatever order
 * is most convenient for it.
 *<P>
 * You can add and remove elements from the multiset. You can access the
 * count for a particular element, as well as retrieve both the total count
 * of all elements (including duplicates) or the count of just the
 * distinct elements. An Enumeration interface is supported, as well as a 
 * mapping operation implemented with a call-back interface.
 * <P>
 * As given, this class contains just prototypes of the methods you
 * must implement along with comments detailing the requirements.
 * There is no implementation, that is your task! Read the comments
 * carefully to make sure that you meet all the necessary specifications.
 * 
 * <STRONG>Important note:</STRONG> In order to facilitate automated
 * testing, it is essential that you not rename any of the public classes, 
 * interfaces, or methods and not change the argument types or order to 
 * any of the public methods. You can change the prototypes by adding
 * exception clauses or the synchronized keyword as needed.
 *
 * @author  Your Name Here!
 * @version 1.1 02/10/99
 */

public class Multiset {

	
	/**
	 * Constructs an empty Multiset.
	 */
	public Multiset()
	{}
	
	
	
    /**
     * Returns the number of elements in this multiset.  This count includes 
     * duplicates, e.g. a multiset containing {A,B,A,A} has 4 total 
     * elements.The operation runs in at most linear time to the number of 
     * distinct elements.
     *
     * @return  the number of elements (including duplicates)
     */
	public int numElements()
	{}

    /**
     * Returns the number of distinct elements in this multiset. Duplicates 
     * are only counted once, e.g. a multiset of {A,B,A,A} has just 2 
     * distinct elements.  This operation runs in at most constant time.
     *
     * @return  the number of distinct elements (excluding duplicates)
     */
	public int numDistinctElements()
	{}
	


    /**
     * Returns the number of times the given element occurs in this multiset.
     * The result is zero if the element doesn't appear at all. This 
     * operation runs in at most linear time to the number of distinct elements.
     *
     * @param      elem   element to count occurrences of
     * @return  the number of occurrences of the object
     */
	public int countOccurrences(Object elem)
	{}

    /**
     * Adds one occurrence of the element to this multiset.  Adding
     * A to the multiset {A,C,B,C} results in {A,C,B,C,A}. Note that since
     * the element ordering is not guaranteed, the implementation is free
     * to arrange the elements in any way, so the result could equivalently
     * have been {A,A,C,B,C}. The element being added cannot be
     * null, if so, the unchecked IllegalArgumentException is thrown.
     * Like all of the standard Java  collections, a reference to 
     * the object is stored, no deep copy or clone is made when adding a 
     * new element. This operation runs in at most linear time to the number of 
     * distinct elements.
     *
     * @param      elem   element to add to multiset
     * @exception  IllegalArgumentException  if elem is null 
     */
	public void add(Object elem)
	{}

    /**
     * Adds a number of occurrences of the element to this multiset.  
     * Adding 3 occurrences of A to {A,C} results in {A,C,A,A,A}
     * (as noted previously, ordering could be different). Exceptions are 
     * raised if the element is null or numOccurrences is negative.
     * Adding zero occurrences is accepted, but makes no changes to the multiset.
     * Like all of the standard Java collections, a reference to the object 
     * is stored, no deep copy or clone is made when adding a new element.
	 * This operation runs in at most linear time to the number of distinct 
	 * elements.
	 *
     * @param      elem   element to add to multiset
     * @param      numOccurrences   number of occurrences to add
     * @exception  IllegalArgumentException  if elem is null or numOccurrences is negative
     */
	public void add(Object elem, int numOccurrences)
	{}

	
	
    /**
     * Removes one occurrence of the element from this multiset. Removing
     * A from the {B,A,C,A} results in {B,C,A} (in some order).
     * If the element doesn't appear at all in the multiset, 
     * no changes are made and no error results. An exception is
     * raised if the element is null. This operation runs in at most 
     * linear time to the number of distinct elements.
     *
     * @param      elem   element to remove from multiset
     * @exception  IllegalArgumentException  if elem is null 
     */
	public void remove(Object elem)
	{}

	
    /**
     * Removes a number of occurrences of the element from this multiset. 
     * Removing 3 occurrences of A from {B,A,A,A,D,A} would give {B,A,D} 
     * (in some order). If there aren't enough occurrences, e.g..
     * trying to remove 10 occurrences of A or 1 occurrence of Z from the
     * multiset {B,A,A,D}, all possible occurrences of the element are
     * removed, and no error results. An exception is raised if the element 
     * is null or if the number of occurrences is negative. This operation 
     * runs in at most linear time to the number of distinct elements.
     *
     * @param      elem   element to remove from multiset
     * @param      numOccurrences   number of occurrences to remove
     * @exception  IllegalArgumentException  if elem is null or numOccurrences is negative
     */
	public void remove(Object elem, int numOccurrences)
	{}


    /**
     * Removes all occurrences of the element from this multiset. 
     * Removing all occurrences of A from {B,A,A,A,D,A} would give 
     * {B,D}. If the element doesn't appear at all in the multiset, 
     * no changes are made and no error results. An exception is
     * raised if the element is null. This operation runs in at most 
     * linear time to the number of distinct elements 
     *
     * @param      elem   element to remove from multiset
     * @exception  IllegalArgumentException  if elem is null
     */
	public void removeAll(Object elem)
	{}

	
		
    /**
     * Returns an enumeration for all the elements (including duplicates)
     * in this multiset. The returned result is an object implementing
     * the Enumeration interface that can be repeatedly messaged to
     * access each element in the multiset one by one. For example, the 
     * Enumeration object returned for the multiset {B,A,B,D,B} would cycle
     * through all 5 elements in some order when repeatedly sent the 
     * <CODE>nextElement</CODE> method. The Enumeration should throw a 
     * NoSuchElementException in the <CODE>nextElement</CODE> method if
     * called when <CODE>hasMoreElements</CODE> has returned false.
     * It should be possible to have more than one ongoing enumeration
     * over the same Multiset (i.e. starting another enumeration
     * while in the middle of one shouldn't cause interference). This
     * method must operate in constant time.
     *
     * @return  an enumeration of the elements in this multiset
     * @see     java.util.Enumeration
     */
	public Enumeration elements()
	{}

    /**
     * Returns an enumeration over the distinct elements (excluding duplicates)
     * in this multiset. The returned result is an object implementing
     * the Enumeration interface that can be repeatedly messaged to
     * access each unique element in the multiset one by one. For example,
     * the Enumeration object returned for the multiset {B,A,B,D,B} would 
     * cycle through the 3 unique elements {B,A,D} in some order when
     * repeatedly sent the <CODE>nextElement</CODE> method. The Enumeration
     * should throw a NoSuchElementException in the <CODE>nextElement</CODE>
     * method if called when <CODE>hasMoreElements</CODE> has returned 
     * false. It should be possible to have more than one ongoing 
     * enumeration over the same Multiset (i.e. starting another enumeration
     * while in the middle of one shouldn't cause interference). This
     * method must operate in constant time.
     *
     * @return  an enumeration of the distinct elements in this multiset
     * @see     java.util.Enumeration
     */
	public Enumeration distinctElements()
	{}

    /**
     * Iterates over the multiset and sends the <CODE>doOneElement</CODE> 
     * message to the Mapping object once for each distinct element 
     * passing the element and its count of occurrences. For example, 
     * map over the multiset {A, B, A, A} would call <CODE>doOneElement</CODE>
     * once on arguments (A, 3) and again on (B, 1). The elements are not
     * visited in any specified order, the implementation is free to do 
     * what is most convenient. This operation runs in at most linear time
     * to the number of distinct elements (not accounting for any
     * time spent in the client's callback method).  
     *  
     * @param  mp   client call-back object implementing Mapping interface
     */	
     public void map(Mapping mp)
	{}

	
    /**
     * Iterates over the distinct elements of the multiset in order as 
     * defined by the comparator, calling the <CODE>doOneElement</CODE> 
     * method once for each distinct element. The Comparator method
     * <CODE>compare</CODE> is called on two elements and their count of 
     * occurrences and returns an integer (negative, 0, or positive) that 
     * establishes the ordering between the two. The mapping operation first
     * calls <CODE>doOneElement</CODE> on the element that was determined to
     * be "before" all the other elements, then onto the nextmost element,
     * and through all the distinct elements finally to the one that was 
     * determined to be "after" all the others. Elements that the callback 
     * reports to be equal can be visited in any order. Mapping in order
     * over the multiset {A,B,A,A,C,C} using a comparator that ordered
     * elements by increasing frequency would call <CODE>doOneElement</CODE>
     * first on (B,1) then on (C,2), and then on (A,3). 
     * <P>It is acceptable for this operation to re-arrange the internal
     * order of the elements if convenient. The entire operation
     * may operate in quadratic time relative to the number of
     * distinct elements (not accounting for any time spent in the
     * client's callback methods). 
     *  
     * @param  mp   client call-back object implementing Mapping interface
     * @param  cmp   client call-back object implementing Comparator interface
     */	
	public void mapInOrder(Mapping mp, Comparator cmp)
	{}


    /**
     * Defines the callback interface used by the Multiset map operations.
     * Objects that implement this interface must provide the implementation
     * for the method used to callback to the client for each element.
	 */
	public interface Mapping {
		public void doOneElement(Object elem, int numOccurrences);
	}

    /**
     * Defines a callback interface used by the Multiset mapInOrder operation.
     * Objects that implement this interface must provide the implementation
     * for the method used to compare two elements to establish their
     * relative ordering. A negative result means element1 comes "before"
     * element2, zero means they are equal, and a positive result means
     * element1 comes "after" element2.
	 */
	public interface Comparator {
		public int compare(Object elem1, int numOccurrences1, 
						   Object elem2, int numOccurrences2);
	}
	

    /**
	 * Convenience class to work with the Multiset map method.
	 * Prints the element along with its occurrence count on a line 
	 * by itself. For example, if the elements were strings, it would
	 * print "Apple (3 occurrences)".  It would be a nice touch to 
	 * match singular/plural on the occurrence, but not essential.
	 */
	public static class PrintElementAndOccurrences
	{}

    /**
	 * Convenience class to work with the Multiset mapInOrder method.
	 * Determines the order of the elements by considering their relative 
	 * frequency. When ordering by increasing frequency, elements with 
	 * lower occurrence counts are considered to be before those that 
	 * are higher.  Elements that have the same frequency are reported 
	 * as equal (and thus, can be mapped in any order).
	 */
	public static class OrderByIncreasingFrequency
	{}
	
    /**
	 * Convenience class to work with the Multiset mapInOrder method.
	 * Determines the order of the elements by considering their relative 
	 * frequency. When ordering by decreasing frequency, elements with 
	 * higher occurrence counts are considered to be before those that 
	 * are lower.  Elements that have the same frequency are reported 
	 * as equal (and thus, can be mapped in any order).
	 */
	public static class OrderByDecreasingFrequency
	{}


}
